pro ssw_track_fov, index, data, 		$
                     iout, dout, 		$
                     ref_helio=ref_helio, 	$
                     ref_date=ref_date, 	$
                     outsize=outsize, 		$
                     debug=debug,arcsec=arcsec, $
                     ref_map=ref_map,           $
                     interactive=interactive    
;+
; NAME:
;    ssw_track_fov
;
; PURPOSE:
;    Extract a sub-field from the SSW-compliant (2D) image data
;    based on the reference time and reference coordinates.
; CALLING SEQUENCE:
;     ssw_track_fov, index, data, outindex, outdata,/interactive 
;-OR- ssw_track_fov, index, data, outindex, outdata,ref_map=ref_map
;-OR- ssw_track_fov, index, data, outindex, outdata, $ 
;        helio=[LAT, LON], date_helio=ssw_std_time, outsize=[nx , ny]
;
; INPUTS:
;    index      - SSW-compliant index structure.
;    data       - 2 or 3D data cube.
;    ref_helio  - Reference latitude and longitude for the alignment.
;    ref_date   - Reference date/time for the heliocentric input.
;                 (This can be an index structure.)
;    outsize    - Dimension of output image in pixels.
;    arcsec     - set for reference in arsecs instead of lon/lat.
;    ref_map    -  a sub field map from the DMZ package
;    interactive - if set, box select from 
;
; OUTPUTS:
;    indexout     - modified index header.
;    datatout     - extracted/aligned data array.  Will be same resolution
;                   as input data.
;
; RESTRICTIONS:
;    This routine relies on getting a SSW standard index vector  
;
; METHOD:
;    Calculates sub-region from ref_helio and ref_date and then 
;    passes the correct sub-region to <extract_arr.pro>.
;
; HISTORY:
;       10-sep-96 MDI_TRACK_FOV Written by G.L. Slater
;       22-Oct-96 - (BNH) - Shamelessly stolen and butchered from GLS
;       16-jan-97 S.L.Freeland (eit/mdi compliant->online)
;       16-sep-97 J. Newmark - added arcsec keyword, handle if off_limb
;        9-Apr-98 S.L.Freeland - bullet proof, add /interactive (DMZ map)
;                                add REF_MAP input, move some stuff outsie
;                                of the loop for vectorization
;-
if not data_chk(index,/struct) or n_params() lt 4then begin 
  box_message,'IDL> ssw_track_fov,index,data,outindex,outdata[options]
  return
endif

num_images = n_elements(index)

if num_images ne data_chk(data,/nimages) then begin 
   box_message,'Mismatch between INDEX,DATA elements'
   return
endif  
iout = index

if keyword_set(interactive) then begin
   box_message,'Calling Mapping Software for sub-region select' 
   index2map,index(0),bytscl(alog10(data(*,*,0)>1)),/sub,ref_map
endif 

if keyword_set(ref_map) then begin 
   if not valid_map(ref_map) then begin 
      box_message,'invalid REF_MAP'
      return
   endif
   ref_date=gt_tagval(ref_map,/time)
   ref_helio=reverse(arcmin2hel(ref_map.xc/60.,ref_map.yc/60.))
   outsize=(size(ref_map.data))(1:2)
endif
;
case 1 of 
   n_elements(outsize) eq 0: outs=[data_chk(data,/nx)*.25,data_chk(data,/ny)*.25]
   n_elements(outsize) eq 1: outs=replicate(outsize(0),2)
   else: outs=outsize(0:1)
endcase

dout = fltarr(outs(0), outs(1), num_images)

; Get the reference time
if (data_chk(ref_date, /struct)) then $
    ref_time = fmt_tim(ref_date) $		; It was an index structure
else $
   ref_time = ref_date				; It was something else

off_limb = 0

if keyword_set(arcsec) then begin
    helio = shift(arcmin2hel(ref_helio(0)/60.,ref_helio(1)/60.,$
               date=ref_date,off_limb=off_limb,/soho),1)
    if not off_limb(0) then ref_helio = helio
endif

if tag_exist(iout,'object') then iout.object = 'partial FOV'
time_img = fmt_tim(index)

for i=0, num_images-1 do begin
  del_t = int2secarr(time_img(i),ref_time)/86400d
  suncenter = [index(i).crpix1, index(i).crpix2]
  xsiz_pix = outs(0)
  ysiz_pix = outs(1)

  if not off_limb(0) then begin
       helio = [ref_helio(0)+diff_rot(del_t(0),ref_helio(1),/synodic), $
          ref_helio(1)]
;
;  Try to ignore the fact that it's not required for 
;  cdelt1 == cdelt2.  
;
     fov_cen = conv_h2p(helio, time_img(i), behind=0, suncenter=suncenter, $
                     pix_size=index.cdelt1, radius=index.solar_r)

   endif else fov_cen = suncenter + ref_helio/index.cdelt1            

  dout(*,*,i) = extract_arr(data(*,*,i), xcen=fov_cen(0), ycen=fov_cen(1), $
                            xsiz=xsiz_pix, ysiz=ysiz_pix)

;
;  Should probably be able to change this in the original FITS header, 
;  But I don't see how...
;
  iout(i).naxis1 = xsiz_pix
  iout(i).naxis2 = ysiz_pix

; remember subfield values are physical postions on array, need to add 1 in 
; x-direction and 20 in y-direction?

  if tag_exist(iout,'p1_x') then begin    ; special update for nonstandard EIT
     sz_data=size(data)
     iout(i).p1_x = (fov_cen(0)-0.5*xsiz_pix+1) + 1 > 0
     iout(i).p2_x = (fov_cen(0)+0.5*xsiz_pix) + 1 < (sz_data(1)-1) + 1
     iout(i).p1_y = (fov_cen(1)-0.5*xsiz_pix+1) + 20 > 0
     iout(i).p2_y = (fov_cen(1)+0.5*xsiz_pix) + 20 < (sz_data(2)-1) + 20
  endif

endfor

end
